'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.setCache = exports.getCache = exports.fetchCache = exports.fetchData = exports.cacheData = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _curry = require('lodash/curry');

var _curry2 = _interopRequireDefault(_curry);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var FIFTEEN_MINUTES = 15;

var apiCache = {};

var cacheData = exports.cacheData = (0, _curry2.default)(function (cacheToUse, minutes, key, data, responseType) {
  var expires = Date.now() + minutes * 60 * 1000;
  cacheToUse[key] = {
    expires: expires,
    data: data,
    responseType: responseType
  };
  return data;
});

var fetchData = exports.fetchData = function fetchData(cacheToUse, key) {
  if (!cacheToUse[key]) return undefined;
  var _cacheToUse$key = cacheToUse[key],
      expires = _cacheToUse$key.expires,
      responseType = _cacheToUse$key.responseType,
      data = _cacheToUse$key.data;

  if (Date.now() >= expires) {
    delete cacheToUse[key];
    return undefined;
  }
  if (responseType === 'promise') return Promise.resolve(data);
  return data;
};

var defaultStringify = function defaultStringify() {
  for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
    args[_key] = arguments[_key];
  }

  return JSON.stringify(args);
};

var expiringCache = (0, _curry2.default)(function (minutes, fn) {
  var keyGen = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultStringify;
  return function () {
    var key = keyGen.apply(undefined, arguments);
    var cache = cacheData(apiCache, minutes, key);

    var existing = fetchData(apiCache, key);
    if (existing) return existing;

    var result = fn.apply(undefined, arguments);

    if (result instanceof Promise) {
      return result.then(function (data) {
        return cache(data, 'promise');
      }).catch(function () {
        delete cache[key];
      });
    }
    if (result instanceof Error) return result;

    return cache(result, typeof result === 'undefined' ? 'undefined' : _typeof(result));
  };
});

var fetchCache = exports.fetchCache = expiringCache(FIFTEEN_MINUTES);

var getCache = exports.getCache = function getCache() {
  var cache = apiCache;
  apiCache = {};
  return cache;
};
var setCache = exports.setCache = function setCache(newCache) {
  if (newCache) apiCache = newCache;
};