'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _v = require('uuid/v4');

var _v2 = _interopRequireDefault(_v);

var _uuidValidate = require('uuid-validate');

var _uuidValidate2 = _interopRequireDefault(_uuidValidate);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var CSRF = 'x-csrf';
var ONE_DAY = 24 * 60 * 60 * 1000;

var shouldValidate = function shouldValidate(req) {
  var method = req.method.toUpperCase();
  return method !== "GET" && method !== "HEAD";
};

var hasValidToken = function hasValidToken(req) {
  var areUUIDs = (0, _uuidValidate2.default)(req.cookies[CSRF], 4) && (0, _uuidValidate2.default)(req.headers[CSRF], 4);
  var areEqual = req.cookies[CSRF] === req.headers[CSRF];
  return areUUIDs && areEqual;
};

var attachCSRF = function attachCSRF(res, token, next) {
  res.header(CSRF, token);
  res.cookie(CSRF, token, { maxAge: ONE_DAY });
  return next();
};

var csrfMiddleware = function csrfMiddleware(req, res, next) {
  var token = req.cookies[CSRF] || (0, _v2.default)();
  if (shouldValidate(req)) {
    if (hasValidToken(req)) {
      attachCSRF(res, token, next);
    } else {
      return next(new Error('INVALID CSRF TOKEN'));
    }
  } else {
    attachCSRF(res, token, next);
  }
};

exports.default = csrfMiddleware;