'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _react = require('react');

var _react2 = _interopRequireDefault(_react);

var _server = require('react-dom/server');

var _reactHelmet = require('react-helmet');

var _reactHelmet2 = _interopRequireDefault(_reactHelmet);

var _reactRedux = require('react-redux');

var _createMemoryHistory = require('history/createMemoryHistory');

var _createMemoryHistory2 = _interopRequireDefault(_createMemoryHistory);

var _naglfar = require('naglfar');

var _Root = require('./Root');

var _Root2 = _interopRequireDefault(_Root);

var _configureStore = require('./configureStore');

var _configureStore2 = _interopRequireDefault(_configureStore);

var _cache = require('./cache');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

var renderCache = {};

var renderHtml = function renderHtml(_ref) {
  var AppComponent = _ref.AppComponent,
      store = _ref.store,
      bundle = _ref.bundle,
      baseCss = _ref.baseCss,
      cssToString = _ref.cssToString;

  var body = (0, _server.renderToString)(_react2.default.createElement(
    _reactRedux.Provider,
    { store: store },
    _react2.default.createElement(AppComponent, null)
  ));

  var rootMarkup = (0, _server.renderToStaticMarkup)(_react2.default.createElement(_Root2.default, {
    head: _reactHelmet2.default.renderStatic(),
    content: body,
    baseStyles: baseCss,
    renderJSStyles: cssToString,
    fetchCache: (0, _cache.getCache)(),
    initialState: store.getState(),
    bundle: bundle
  }));

  return '<!doctype html>\n' + rootMarkup;
};

var resolveRoute = function resolveRoute(_ref2) {
  var path = _ref2.path,
      res = _ref2.res,
      _ref2$cachePerUrl = _ref2.cachePerUrl,
      cachePerUrl = _ref2$cachePerUrl === undefined ? false : _ref2$cachePerUrl,
      config = _objectWithoutProperties(_ref2, ['path', 'res', 'cachePerUrl']);

  if (cachePerUrl) {
    var cachedHtml = (0, _cache.fetchData)(renderCache, path);
    if (cachedHtml) return res.status(200).send(cachedHtml);
  }
  (0, _naglfar.resolveLocation)(path, config.store.dispatch).then(function (_ref3) {
    var status = _ref3.status,
        url = _ref3.url;

    if (url) return res.redirect(status, url);
    var html = renderHtml(config);
    cachePerUrl && (0, _cache.cacheData)(renderCache, 8 * 60, path, html, typeof html === 'undefined' ? 'undefined' : _typeof(html));
    res.status(status).send(html);
  });
};

exports.default = function (config) {
  return function (req, res) {
    resolveRoute(_extends({}, config, {
      res: res,
      cachePerUrl: res.locals.initialState ? false : config.cachePerUrl,
      path: req.url,
      store: (0, _configureStore2.default)((0, _createMemoryHistory2.default)({ initialEntries: [req.url] }), res.locals.initialState)
    }));
  };
};