'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.redirectEndpoint = exports.downloadEndpoint = exports.htmlEndpoint = exports.postEndpoint = exports.getEndpoint = exports.attachMiddleware = undefined;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _webpackDevMiddleware = require('webpack-dev-middleware');

var _webpackDevMiddleware2 = _interopRequireDefault(_webpackDevMiddleware);

var _webpackHotMiddleware = require('webpack-hot-middleware');

var _webpackHotMiddleware2 = _interopRequireDefault(_webpackHotMiddleware);

var _bodyParser = require('body-parser');

var _bodyParser2 = _interopRequireDefault(_bodyParser);

var _cookieParser = require('cookie-parser');

var _cookieParser2 = _interopRequireDefault(_cookieParser);

var _compression = require('compression');

var _compression2 = _interopRequireDefault(_compression);

var _v = require('uuid/v4');

var _v2 = _interopRequireDefault(_v);

var _renderMiddleware = require('./lib/renderMiddleware');

var _renderMiddleware2 = _interopRequireDefault(_renderMiddleware);

var _csrfMiddleware = require('./lib/csrfMiddleware');

var _csrfMiddleware2 = _interopRequireDefault(_csrfMiddleware);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

global.ENVIRONMENT = process.env.NODE_ENV || 'default';
process.on('unhandledRejection', function (reason, p) {
  if (reason.stack) console.error(reason.stack);else console.error('Unhandled Rejection at: Promise ', p, ' reason: ', reason);
});

var server = (0, _express2.default)();

server.set('trust proxy', 'loopback');
server.use((0, _compression2.default)());
server.use(_bodyParser2.default.json());
server.use((0, _cookieParser2.default)());
server.use(_csrfMiddleware2.default);
var attachMiddleware = exports.attachMiddleware = function attachMiddleware(middleware) {
  server.use(middleware);
};

var defaultMiddleware = function defaultMiddleware(req, res, next) {
  return next();
};
var normalizedPromises = function normalizedPromises(routePromise, handleSuccess) {
  return function (req, res) {
    routePromise(_extends({}, req.params, req.body, req.query, res.locals, {
      cookies: req.cookies
    })).then(handleSuccess(res), function (error) {
      return res.status(500).json(error);
    });
  };
};
var sendJSON = function sendJSON(res) {
  return function (data) {
    return res.status(200).json(data);
  };
};
var sendHTML = function sendHTML(res) {
  return function (data) {
    return res.status(200).send(data);
  };
};
var sendDownload = function sendDownload(res) {
  return function (_ref) {
    var url = _ref.url,
        fileName = _ref.fileName,
        done = _ref.done;
    return res.status(200).download(url, fileName || url, done);
  };
};
var sendRedirect = function sendRedirect(res) {
  return function (_ref2) {
    var url = _ref2.url;
    return res.redirect(url);
  };
};

var createEndpoint = function createEndpoint(method) {
  var successHandler = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : sendJSON;
  return function (endpoint, promise) {
    var middleware = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : defaultMiddleware;

    server[method](endpoint, middleware, normalizedPromises(promise, successHandler));
  };
};

var getEndpoint = exports.getEndpoint = createEndpoint('get');
var postEndpoint = exports.postEndpoint = createEndpoint('post');
var htmlEndpoint = exports.htmlEndpoint = createEndpoint('get', sendHTML);
var downloadEndpoint = exports.downloadEndpoint = createEndpoint('get', sendDownload);
var redirectEndpoint = exports.redirectEndpoint = createEndpoint('get', sendRedirect);

// options: {
//   AppComponent,
//   port,
//   config, 
//   baseCss, 
//   cssToString,
//   cachePerUrl
// }

exports.default = function (_ref3) {
  var AppComponent = _ref3.AppComponent,
      port = _ref3.port,
      config = _ref3.config,
      options = _objectWithoutProperties(_ref3, ['AppComponent', 'port', 'config']);

  port = port || process.env.PORT || 3000;

  if (process.env.NODE_ENV !== 'production') {
    if (!config) throw new Error('Dev config not supplied for development server');
    var compiler = (0, _webpack2.default)(_extends({}, config, {
      output: _extends({}, config.output, {
        path: _path2.default.resolve(__dirname, '../../' + config.output.path)
      })
    }));

    server.use((0, _webpackDevMiddleware2.default)(compiler, {
      publicPath: config.output.publicPath,
      stats: {
        colors: true,
        hash: false,
        timings: true,
        chunks: false,
        chunkModules: false,
        modules: false
      }
    }));
    server.use((0, _webpackHotMiddleware2.default)(compiler));
  }

  server.use(_express2.default.static(config.output.path));

  server.get('*', (0, _renderMiddleware2.default)(_extends({ AppComponent: AppComponent, bundle: config.output.filename }, options)));

  server.listen(port, function (err) {
    if (err) {
      console.error(err);
      return;
    }
    console.info('\u26A1\u26A1\u26A1 Server running on http://localhost:' + port + '/');
  });

  return server;
};